@@ -87,7 +87,15 @@ void print_mapping(const int pos_on_fragment, const char * fragment, const char
 
 
 
-void feed_coherent_positions(p_fragment_info the_starter, const int start, const int length_read, const int qual, char *quality, int start_on_read, int read_file){
+void feed_coherent_positions(p_fragment_info the_starter, //the fragment/path
+        const int start, //where the read begin on the fragment
+        const int length_read,
+        const int qual,
+        char *quality,
+        int start_on_read, //where the read seed starts in the read
+        int read_file,
+        char* whereReadMaps
+        ) {
     int i, start_on_starter, stop_on_starter;
 	if(start<0) start_on_starter=0; else start_on_starter=start;
 	if(start+length_read<strlen(the_starter->w)) stop_on_starter=start+length_read; else stop_on_starter=strlen(the_starter->w);
@@ -117,21 +125,37 @@ void feed_coherent_positions(p_fragment_info the_starter, const int start, const
 
         char overlap_AS=0;
         char overlap_SB=0;
+        char overlap_S = 0;
         
-        if(start_on_starter<=size_seeds-min_overlap && stop_on_starter>=size_seeds+min_overlap) overlap_AS=1;
-        const int pos_SB = strlen(the_starter->w)-(size_seeds);
-        if(start_on_starter<=pos_SB-min_overlap && stop_on_starter>=pos_SB+min_overlap) overlap_SB=1;
-        if(overlap_AS) the_starter->nb_reads_overlapping_AS[read_file]++;
-        if(overlap_SB) the_starter->nb_reads_overlapping_SB[read_file]++;
-        if(overlap_AS && overlap_SB) the_starter->nb_reads_overlapping_both_AS_and_SB[read_file]++;
-        if(start_on_starter>size_seeds-min_overlap && stop_on_starter<pos_SB+min_overlap) the_starter->nb_reads_fully_in_S[read_file]++;
-    }
-    else{// lower path
-      if(stop_on_starter-start_on_starter>size_seeds+min_overlap)
+        if (start_on_starter <= kisspliceKmerSize - min_overlap && stop_on_starter >= kisspliceKmerSize + min_overlap) overlap_AS = 1;
+        const int pos_SB = strlen(the_starter->w)- kisspliceKmerSize;
+        if (start_on_starter <= pos_SB - min_overlap && stop_on_starter >= pos_SB + min_overlap) overlap_SB = 1;
+        if (overlap_AS) the_starter->nb_reads_overlapping_AS[read_file]++;
+        if (overlap_SB) the_starter->nb_reads_overlapping_SB[read_file]++;
+        if (overlap_AS && overlap_SB) the_starter->nb_reads_overlapping_both_AS_and_SB[read_file]++;
+        if (start_on_starter > kisspliceKmerSize - min_overlap && stop_on_starter < pos_SB + min_overlap) overlap_S = 1;
+        if (overlap_S) the_starter->nb_reads_fully_in_S[read_file]++;
+        //////////////////////////////////////////////////////        
+        
+        whereReadMaps[0] = overlap_AS;
+        whereReadMaps[1] = overlap_SB;
+        whereReadMaps[2] = overlap_AS && overlap_SB;
+        whereReadMaps[3] = overlap_S;
+        whereReadMaps[4] = 0;
+        
+    } else {// lower path
+        char overlap_AB = 0;
+        if (stop_on_starter - start_on_starter >= kisspliceKmerSize + min_overlap) {
+            overlap_AB = 1;
             the_starter->nb_reads_overlapping_both_AS_and_SB[read_file]++;
-
+        }
+        
+        whereReadMaps[0] = 0;
+        whereReadMaps[1] = 0;
+        whereReadMaps[2] = 0;
+        whereReadMaps[3] = 0;
+        whereReadMaps[4] = overlap_AB;
     }
-    
 #endif
     
 #ifdef CHARQUAL
@@ -437,9 +461,13 @@ float read_coherence (gzFile reads_file,
                             printf("SUCCESS %d %d \n", pwi, value->a);
                             print_mapping(pwi,all_starters[value->a]->w ,read); //DEB
 #endif
-                            feed_coherent_positions(all_starters[value->a], pwi, (int)strlen(read), qual, quality, i, read_file);
-                            if( sam_out ) fprintf(sam_out,"%d %d %s\t%s\tC%d\t%d\t%s", value->a, value->b, all_starters[value->a]->comment, read, read_file+1, pwi, readid);
-                            
+                            char whereReadMaps[5] = {0, 0, 0, 0, 0};
+                            feed_coherent_positions(all_starters[value->a], pwi, (int) strlen(read), qual, quality, i, read_file, whereReadMaps);
+                            if (sam_out)
+                                fprintf(sam_out, "%d %d %s\t%s\tC%d\t%d\t%s\t%d\t%d\t%d\t%d\t%d\n",
+                                        value->a, value->b, all_starters[value->a]->comment, read, read_file + 1, pwi, readid,
+                                        (int)whereReadMaps[0], (int)whereReadMaps[1], (int)whereReadMaps[2], (int)whereReadMaps[3],
+                                        (int)whereReadMaps[4]);                            
 #ifdef DEBUG_QUALITY
                             
                             if ( qual )
