@@ -65,9 +65,9 @@ char * getVersion(){
 void print_usage_and_exit(char * name){
 	fprintf (stderr, "NAME\nkissReads, version %s\n", getVersion());
 #ifdef INPUT_FROM_KISSPLICE
-	fprintf (stderr, "\nSYNOPSIS\n%s <toCheck.fasta> <readsC1.fasta> [<readsC2.fasta> [<readsC3.fasta] ...] [-k value] [-c value] [-d value] [-O value] [-l value] [-o name] [-u name] [-i index_stride] [-m align_file] [-p] [-s] [-f] [-h] \n", name);
+	fprintf (stderr, "\nSYNOPSIS\n%s <toCheck.fasta> <readsC1.fasta> [<readsC2.fasta> [<readsC3.fasta] ...] [-k value] [-S value] [-c value] [-d value] [-O value] [-l value] [-o name] [-u name] [-i index_stride] [-m align_file] [-p] [-s] [-f] [-h] \n", name);
 #else
-    fprintf (stderr, "\nSYNOPSIS\n%s <toCheck.fasta> <readsC1.fasta> [<readsC2.fasta> [<readsC3.fasta] ...] [-k value] [-c value] [-d value] [-O value] [-o name] [-u name] [-n] [-I] [-i index_stride] [-m align_file] [-p] [-s] [-f] [-h] \n", name);
+    fprintf (stderr, "\nSYNOPSIS\n%s <toCheck.fasta> <readsC1.fasta> [<readsC2.fasta> [<readsC3.fasta] ...] [-S value] [-c value] [-d value] [-O value] [-o name] [-u name] [-n] [-I] [-i index_stride] [-m align_file] [-p] [-s] [-f] [-h] \n", name);
 #endif// INPUT_FROM_KISSPLICE
 	fprintf (stderr, "\nDESCRIPTION\n");
 	fprintf (stderr, "Checks for each sequence contained into the toCheck.fasta if\n");
@@ -81,8 +81,11 @@ void print_usage_and_exit(char * name){
 	fprintf (stderr, "  Any other situation (s not coherent with any): couple non read coherent, not outputed \n");
     
 	fprintf (stderr, "\nOPTIONS\n");
-	fprintf (stderr, "\t -k size_seed: will use seeds of length size_seed. Default: 25.\n");
-    fprintf (stderr, "\t -O minimal_read_overlap: a read can be mapped if its overlap is a least \"minimal_read_overlap\". Default: k\n");
+#ifdef INPUT_FROM_KISSPLICE        
+	fprintf (stderr, "\t -k k-mer size: this is the k-mer size used in KisSplice.\n");
+#endif        
+	fprintf (stderr, "\t -S size_seed: will use seeds of length size_seed. Default: 25.\n");
+    fprintf (stderr, "\t -O minimal_read_overlap: a read can be mapped if its overlap is a least \"minimal_read_overlap\". Default: S\n");
 	fprintf (stderr, "\t -c min_coverage: a sequence is covered by at least min_coverage coherent reads. Default: 2\n");
     fprintf (stderr, "\t -d max_substitutions: Maximal number of substitutions authorized between a read and a fragment. Note that no substitution is allowed on the central position while anaylizing the kissnp output. Default: 1.\n");
 #ifdef INPUT_FROM_KISSPLICE
@@ -128,6 +131,7 @@ int main(int argc, char **argv) {
     char input_only_upper=0; // By default: all characters (upper or lower) are read
     int number_paths_per_event=1; // By default (generic usage) each event is composed by a unique sequence.
 #ifdef INPUT_FROM_KISSPLICE
+	kisspliceKmerSize = -1;
     min_overlap=3; // default value.
     countingOption = 0 ; // default value
     no_subsutitution_on_central_position=0; // just to be sure, as normaly this is the default behaviour
@@ -182,9 +186,9 @@ int main(int argc, char **argv) {
 	while (1)
 	{
 #ifdef INPUT_FROM_KISSPLICE
-        int temoin = getopt (argc-number_of_read_sets-1, &argv[number_of_read_sets+1], "c:d:k:O:o:u:q:m:i:fps-:j:l:t:");
+        int temoin = getopt(argc - number_of_read_sets - 1, &argv[number_of_read_sets + 1], "c:d:k:S:O:o:u:q:m:i:fps-:j:l:t:");
 #else
-        int temoin = getopt (argc-number_of_read_sets-1, &argv[number_of_read_sets+1], "c:d:k:O:o:u:q:m:i:fpsnI-:t:");
+        int temoin = getopt(argc - number_of_read_sets - 1, &argv[number_of_read_sets + 1], "c:d:S:O:o:u:q:m:i:fpsnI-:t:");
 #endif //INPUT_FROM_KISSPLICE
 		if (temoin == -1){
 			break;
@@ -199,6 +203,9 @@ int main(int argc, char **argv) {
                 only_print=1;
                 break;
 #ifdef INPUT_FROM_KISSPLICE
+            case 'k':
+                kisspliceKmerSize = atoi(optarg);
+                break;
             case 'l':
                 min_overlap=atoi(optarg);
                 break;
@@ -254,7 +261,7 @@ int main(int argc, char **argv) {
                 minimal_read_overlap=atoi(optarg);
                 break;
                 
-            case 'k':
+            case 'S':
                 size_seeds=atoi(optarg);
                 
                 if(size_seeds<10){
@@ -278,7 +285,7 @@ int main(int argc, char **argv) {
                     fprintf(stderr,"cannot open %s for writing sam like results, exit\n", optarg);
                     exit(1);
                 }
-                fprintf(sam_out,"SNP_header\tmapped_read\tid_read_file\tposition_where_read_is_mapped\tread_id\n");
+                fprintf(sam_out, "SNP_header\tmapped_read\tid_read_file\tposition_where_read_is_mapped\tread_id\tAS\tSB\tASSB\tS\tAB\n");
 
                 
                 break;
@@ -317,6 +324,12 @@ int main(int argc, char **argv) {
 
     if (only_print) uncoherent_out = coherent_out;
     
+#ifdef INPUT_FROM_KISSPLICE    
+    if (kisspliceKmerSize == -1) {
+        fprintf(stderr, "Sorry, you must specify -k parameter, exit\n");
+        exit(1);
+    }
+#endif    
     
     if(!silent){
         printf("This is kissreads, version %s\n", getVersion());
@@ -342,7 +355,10 @@ int main(int argc, char **argv) {
         
         
         printf("\tMAPPING PARAMETERS\n");
-        printf("\t *(-k) Size seeds %d\n", size_seeds);
+#ifdef INPUT_FROM_KISSPLICE        
+        printf("\t *(-k) k-mer size %d\n", kisspliceKmerSize);
+#endif
+        printf("\t *(-S) Size seeds %d\n", size_seeds);
         printf("\t *(-i) Index Stride %d (will index a seed each %d positions)\n", index_stride, index_stride);
         printf("\t *(-O) minimal_read_overlap %d (a read is mapped on a fragment if the overlap is at least %d positions)\n", minimal_read_overlap,minimal_read_overlap);
         printf("\t \t Note that minimal_read_overlap is at least equal to size_seeds\n");
